package com.jidesoft.docking;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.KeyboardFocusManager;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;

import javax.swing.JComponent;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;
import javax.swing.border.Border;

import com.jidesoft.swing.Resizable;
import com.jidesoft.swing.ResizablePanel;
import com.jidesoft.utils.PortingUtils;

public class ExtendedDialogFloatingContainer extends DialogFloatingContainer {

    private ResizablePanel m_resizablePanel;

    public ExtendedDialogFloatingContainer(
            DockingManager dockingManager,
            FloatingContainerManager floatingContainerManager,
            Frame frame) throws HeadlessException {
        super(dockingManager, floatingContainerManager, frame);
    }

    public ExtendedDialogFloatingContainer(
            DockingManager dockingManager,
            FloatingContainerManager floatingContainerManager,
            Dialog dialog) throws HeadlessException {
        super(dockingManager, floatingContainerManager, dialog);
    }

    @Override
    protected JRootPane createRootPane() {
        var rootPane = new DockingRootPane();
        rootPane.setOpaque(true);
        return rootPane;
    }

    @Override
    protected void initComponents() {
        setModal(false);
        setUndecorated(true);

        m_resizablePanel = new ResizablePanel() {
            @Override
            protected Resizable createResizable() {
                return new Resizable(this) {
                    @Override
                    public void resizing(int resizeDir, int newX, int newY, int newW, int newH) {
                        Container container = ExtendedDialogFloatingContainer.this.getContentPane();
                        PortingUtils.setPreferredSize(container, new Dimension(newW, newH));
                        if (ExtendedDialogFloatingContainer.this.isUndecorated()) {
                            ExtendedDialogFloatingContainer.this.setBounds(newX, newY, newW, newH);
                        }
                        ExtendedDialogFloatingContainer.this.resizing();
                    }

                    @Override
                    public void beginResizing(int resizeCorner) {
                        super.beginResizing(resizeCorner);
                        ExtendedDialogFloatingContainer.this.beginResizing();
                    }

                    @Override
                    public void endResizing(int resizeCorner) {
                        super.endResizing(resizeCorner);
                        ExtendedDialogFloatingContainer.this.endResizing();
                    }

                    @Override
                    public boolean isTopLevel() {
                        return true;
                    }
                };
            }

            @Override
            protected boolean processKeyBinding(
                    KeyStroke ks, KeyEvent e, int condition, boolean pressed) {
                boolean processed = super.processKeyBinding(ks, e, condition, pressed);
                if (processed
                        || e.isConsumed()
                        || !ExtendedDialogFloatingContainer.this.isRoutingKeyStrokes()) {
                    return processed;
                }

                // check if the root pane of the source component has any registered action
                if (e.getSource() instanceof JComponent component) {
                    if (component.getRootPane() instanceof DockingRootPane rootPane) {
                        processed = rootPane.processKeyBinding(
                                ks, e, JComponent.WHEN_IN_FOCUSED_WINDOW, pressed);
                    }
                }

                if (processed || e.isConsumed()) {
                    return processed;
                }

                Component routingParent = ExtendedDialogFloatingContainer.this.getRoutingComponent();
                if (routingParent == null) {
                    return false;
                }

                KeyboardFocusManager.getCurrentKeyboardFocusManager()
                        .redispatchEvent(routingParent, e);
                return (e.isConsumed());
            }
        };

        setContentPane(m_resizablePanel);

        // make sure the content pane resized along with the window.
        addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                m_resizablePanel.setSize(getSize());
            }
        });
    }

    @Override
    public void setBorder(Border border) {
        m_resizablePanel.setBorder(border);
    }

    @Override
    public Border getBorder() {
        return m_resizablePanel.getBorder();
    }

    @Override
    public Resizable getResizable() {
        return m_resizablePanel.getResizable();
    }
}