package com.jidesoft.docking;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.KeyboardFocusManager;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;

import javax.swing.JComponent;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;

import com.jidesoft.swing.Resizable;
import com.jidesoft.swing.ResizablePanel;
import com.jidesoft.utils.PortingUtils;

public class ExtendedFrameFloatingContainer extends FrameFloatingContainer {

    public ExtendedFrameFloatingContainer(DockingManager dm, FloatingContainerManager fcm)
            throws HeadlessException {
        super(dm, fcm);
    }

    @Override
    protected JRootPane createRootPane() {
        var rootPane = new DockingRootPane();
        rootPane.setOpaque(true);
        return rootPane;
    }

    @Override
    protected void initComponents() {
        setUndecorated(true);

        _resizablePanel = new ResizablePanel() {
            @Override
            protected Resizable createResizable() {
                return new Resizable(this) {
                    @Override
                    public void resizing(int resizeDir, int newX, int newY, int newW, int newH) {
                        Container container = ExtendedFrameFloatingContainer.this.getContentPane();
                        PortingUtils.setPreferredSize(container, new Dimension(newW, newH));
                        if (ExtendedFrameFloatingContainer.this.isUndecorated()) {
                            ExtendedFrameFloatingContainer.this.setBounds(newX, newY, newW, newH);
                        }
                        ExtendedFrameFloatingContainer.this.resizing();
                    }

                    @Override
                    public void beginResizing(int resizeCorner) {
                        super.beginResizing(resizeCorner);
                        ExtendedFrameFloatingContainer.this.beginResizing();
                    }

                    @Override
                    public void endResizing(int resizeCorner) {
                        super.endResizing(resizeCorner);
                        ExtendedFrameFloatingContainer.this.endResizing();
                    }

                    @Override
                    public boolean isTopLevel() {
                        return true;
                    }
                };
            }

            @Override
            protected boolean processKeyBinding(
                    KeyStroke ks, KeyEvent e, int condition, boolean pressed) {
                boolean processed = super.processKeyBinding(ks, e, condition, pressed);
                if (processed
                        || e.isConsumed()
                        || !ExtendedFrameFloatingContainer.this.isRoutingKeyStrokes()) {
                    return processed;
                }

                // check if the root pane of the source component has any registered action
                if (e.getSource() instanceof JComponent component) {
                    if (component.getRootPane() instanceof DockingRootPane rootPane) {
                        processed = rootPane.processKeyBinding(
                                ks, e, JComponent.WHEN_IN_FOCUSED_WINDOW, pressed);
                    }
                }

                if (processed || e.isConsumed()) {
                    return processed;
                }

                Component routingParent = ExtendedFrameFloatingContainer.this.getRoutingComponent();
                if (routingParent == null) {
                    return false;
                }

                KeyboardFocusManager.getCurrentKeyboardFocusManager()
                        .redispatchEvent(routingParent, e);
                return (e.isConsumed());
            }
        };

        setContentPane(_resizablePanel);

        // make sure the content pane resized along with the window.
        addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                _resizablePanel.setSize(getSize());
            }
        });
    }
}